@@ -34,6 +34,7 @@ from ..exceptions import (
     CredentialNoAcTimeValueException,
     CredentialNoBiliJctException,
     CredentialNoBuvid3Exception,
+    CredentialNoBuvid4Exception,
     CredentialNoDedeUserIDException,
     CredentialNoSessdataException,
     ExClimbWuzhiException,
@@ -60,7 +61,7 @@ class RequestLog(AsyncEvent):
             )
             self.logger.addHandler(handler)
         self.__on = False
-        self.__on_events = [
+        self.__on_events: List[str] = [
             "API_REQUEST",
             "API_RESPONSE",
             "ANTI_SPIDER",
@@ -69,42 +70,42 @@ class RequestLog(AsyncEvent):
             "WS_SEND",
             "WS_CLOSE",
         ]
-        self.__ignore_events = []
+        self.__ignore_events: List[str] = []
         self.add_event_listener("__ALL__", self.__handle_events)
 
-    def get_on_events(self) -> dict:
+    def get_on_events(self) -> List[str]:
         """
         获取日志输出支持的事件类型
 
         Returns:
-            dict: 日志输出支持的事件类型
+            List[str]: 日志输出支持的事件类型
         """
         return self.__on_events
 
-    def set_on_events(self, events: dict) -> None:
+    def set_on_events(self, events: List[str]) -> None:
         """
         设置日志输出支持的事件类型
 
         Args:
-            events (dict): 日志输出支持的事件类型
+            events (List[str]): 日志输出支持的事件类型
         """
         self.__on_events = events
 
-    def get_ignore_events(self) -> dict:
+    def get_ignore_events(self) -> List[str]:
         """
         获取日志输出排除的事件类型
 
         Returns:
-            dict: 日志输出排除的事件类型
+            List[str]: 日志输出排除的事件类型
         """
         return self.__ignore_events
 
-    def set_ignore_events(self, events: dict) -> None:
+    def set_ignore_events(self, events: List[str]) -> None:
         """
         设置日志输出排除的事件类型
 
         Args:
-            events (dict): 日志输出排除的事件类型
+            events (List[str]): 日志输出排除的事件类型
         """
         self.__ignore_events = events
 
@@ -175,8 +176,10 @@ Events:
 
 CallbackData: 描述 (str) 数据 (dict)
 
+示例：
+
 ``` python
-@request_log.on("__ALL__")
+@request_log.on("REQUEST")
 async def handle(desc: str, data: dict) -> None:
     print(desc, data)
 ```
@@ -211,8 +214,10 @@ Events:
 
 CallbackData: 描述 (str) 数据 (dict)
 
+示例：
+
 ``` python
-@request_log.on("__ALL__")
+@request_log.on("REQUEST")
 async def handle(desc: str, data: dict) -> None:
     print(desc, data)
 ```
@@ -242,11 +247,15 @@ class RequestSettings:
             "trust_env": True,
         }
         self.__wbi_retry_times = 3
+        self.__enable_auto_buvid = True
+        self.__enable_bili_ticket = False
 
     def get(self, name: str) -> Any:
         """
         获取某项设置
 
+        不可用于 `wbi_retry_times` `enable_auto_buvid` `enable_bili_ticket`
+
         默认设置名称：`proxy` `timeout` `verify_ssl` `trust_env`
 
         Args:
@@ -257,10 +266,12 @@ class RequestSettings:
         """
         return self.__settings[name]
 
-    def set(self, name: str, value: str) -> Any:
+    def set(self, name: str, value: Any) -> None:
         """
         设置某项设置
 
+        不可用于 `wbi_retry_times` `enable_auto_buvid` `enable_bili_ticket`
+
         默认设置名称：`proxy` `timeout` `verify_ssl` `trust_env`
 
         Args:
@@ -363,10 +374,48 @@ class RequestSettings:
         """
         self.__wbi_retry_times = wbi_retry_times
 
+    def get_enable_auto_buvid(self) -> bool:
+        """
+        获取设置的是否自动生成 buvid
+
+        Returns:
+            bool: 是否自动生成 buvid. Defaults to True.
+        """
+        return self.__enable_auto_buvid
+
+    def set_enable_auto_buvid(self, enable_auto_buvid: bool) -> None:
+        """
+        设置是否自动生成 buvid
+
+        Args:
+            enable_auto_buvid (bool): 是否自动生成 buvid.
+        """
+        self.__enable_auto_buvid = enable_auto_buvid
+
+    def get_enable_bili_ticket(self) -> bool:
+        """
+        获取设置的是否使用 bili_ticket
+
+        Returns:
+            bool: 是否使用 bili_ticket. Defaults to True.
+        """
+        return self.__enable_bili_ticket
+
+    def set_enable_bili_ticket(self, enable_bili_ticket: bool) -> None:
+        """
+        设置是否使用 bili_ticket
+
+        Args:
+            enable_bili_ticket (bool): 是否使用 bili_ticket.
+        """
+        self.__enable_bili_ticket = enable_bili_ticket
+
     def get_all(self) -> dict:
         """
         获取目前所有的设置项
 
+        不可用于 `wbi_retry_times` `enable_auto_buvid` `enable_bili_ticket`
+
         Returns:
             dict: 所有的设置项
         """
@@ -941,6 +990,10 @@ def get_selected_client() -> Tuple[str, Type[BiliAPIClient]]:
     Returns:
         Tuple[str, Type[BiliAPIClient]]: 第 0 项为客户端名称，第 1 项为对应的类
     """
+    if selected_client == "":
+        raise ArgsException(
+            "尚未安装第三方请求库或未注册自定义第三方请求库。\n$ pip3 install (curl_cffi|httpx|aiohttp)"
+        )
     return selected_client, sessions[selected_client]
 
 
@@ -951,6 +1004,10 @@ def get_available_settings() -> List[str]:
     Returns:
         List[str]: 支持的设置项名称
     """
+    if selected_client == "":
+        raise ArgsException(
+            "尚未安装第三方请求库或未注册自定义第三方请求库。\n$ pip3 install (curl_cffi|httpx|aiohttp)"
+        )
     return client_settings[selected_client]
 
 
@@ -1062,6 +1119,7 @@ class Credential:
         sessdata: Union[str, None] = None,
         bili_jct: Union[str, None] = None,
         buvid3: Union[str, None] = None,
+        buvid4: Union[str, None] = None,
         dedeuserid: Union[str, None] = None,
         ac_time_value: Union[str, None] = None,
     ) -> None:
@@ -1075,6 +1133,8 @@ class Credential:
 
             buvid3     (str | None, optional): 浏览器 Cookies 中的 BUVID3 字段值. Defaults to None.
 
+            buvid4     (str | None, optional): 浏览器 Cookies 中的 BUVID4 字段值. Defaults to None.
+
             dedeuserid (str | None, optional): 浏览器 Cookies 中的 DedeUserID 字段值. Defaults to None.
 
             ac_time_value (str | None, optional): 浏览器 Cookies 中的 ac_time_value 字段值. Defaults to None.
@@ -1088,6 +1148,7 @@ class Credential:
         )
         self.bili_jct = bili_jct
         self.buvid3 = buvid3
+        self.buvid4 = buvid4
         self.dedeuserid = dedeuserid
         self.ac_time_value = ac_time_value
 
@@ -1101,6 +1162,7 @@ class Credential:
         cookies = {
             "SESSDATA": self.sessdata if self.sessdata else "",
             "buvid3": self.buvid3 if self.buvid3 else "",
+            "buvid4": self.buvid4 if self.buvid4 else "",
             "bili_jct": self.bili_jct if self.bili_jct else "",
             "ac_time_value": self.ac_time_value if self.ac_time_value else "",
         }
@@ -1116,7 +1178,7 @@ class Credential:
         Returns:
             bool: 是否提供 dedeuserid。
         """
-        return self.dedeuserid is not None and self.sessdata != ""
+        return self.dedeuserid is not None and self.dedeuserid != ""
 
     def has_sessdata(self) -> bool:
         """
@@ -1134,7 +1196,7 @@ class Credential:
         Returns:
             bool: 是否提供 bili_jct。
         """
-        return self.bili_jct is not None and self.sessdata != ""
+        return self.bili_jct is not None and self.bili_jct != ""
 
     def has_buvid3(self) -> bool:
         """
@@ -1143,7 +1205,16 @@ class Credential:
         Returns:
             bool: 是否提供 buvid3
         """
-        return self.buvid3 is not None and self.sessdata != ""
+        return self.buvid3 is not None and self.buvid3 != ""
+
+    def has_buvid4(self) -> bool:
+        """
+        是否提供 buvid4
+
+        Returns:
+            bool: 是否提供 buvid4
+        """
+        return self.buvid4 is not None and self.buvid4 != ""
 
     def has_ac_time_value(self) -> bool:
         """
@@ -1152,7 +1223,7 @@ class Credential:
         Returns:
             bool: 是否提供 ac_time_value
         """
-        return self.ac_time_value is not None and self.sessdata != ""
+        return self.ac_time_value is not None and self.ac_time_value != ""
 
     def raise_for_no_sessdata(self):
         """
@@ -1175,6 +1246,13 @@ class Credential:
         if not self.has_buvid3():
             raise CredentialNoBuvid3Exception()
 
+    def raise_for_no_buvid4(self):
+        """
+        没有提供 buvid3 时抛出异常。
+        """
+        if not self.has_buvid4():
+            raise CredentialNoBuvid4Exception()
+
     def raise_for_no_dedeuserid(self):
         """
         没有提供 DedeUserID 时抛出异常。
@@ -1232,12 +1310,13 @@ class Credential:
         c.sessdata = cookies.get("SESSDATA")
         c.bili_jct = cookies.get("bili_jct")
         c.buvid3 = cookies.get("buvid3")
+        c.buvid3 = cookies.get("buvid4")
         c.dedeuserid = cookies.get("DedeUserID")
         c.ac_time_value = cookies.get("ac_time_value")
         return c
 
     def __str__(self):
-        return f"SESSDATA: {self.sessdata}; bili_jct: {self.bili_jct}; buvid3: {self.buvid3}; DedeUserID: {self.dedeuserid}; ac_time_value: {self.ac_time_value}"
+        return f"SESSDATA: {self.sessdata}; bili_jct: {self.bili_jct}; buvid3: {self.buvid3}; buvid4: {self.buvid4}; DedeUserID: {self.dedeuserid}; ac_time_value: {self.ac_time_value}"
 
 
 """
@@ -1841,12 +1920,13 @@ async def _get_bili_ticket(credential: Optional[Credential] = None) -> str:
 __buvid3 = ""
 __buvid4 = ""
 __bili_ticket = ""
+__bili_ticket_expires = 0
 __wbi_mixin_key = ""
 
 
 def refresh_buvid() -> None:
     """
-    刷新 buvid3 和 buvid4
+    刷新模块自动生成的 buvid3 和 buvid4
     """
     global __buvid3, __buvid4
     __buvid3 = ""
@@ -1857,13 +1937,14 @@ def refresh_bili_ticket() -> None:
     """
     刷新 bili_ticket
     """
-    global __bili_ticket
+    global __bili_ticket, __bili_ticket_expires
     __bili_ticket = ""
+    __bili_ticket_expires = 0
 
 
-def refresh_wbi_mixin_key() -> None:
+def recalculate_wbi() -> None:
     """
-    刷新 wbi mixin key
+    重新计算 wbi 的参数
     """
     global __wbi_mixin_key
     __wbi_mixin_key = ""
@@ -1890,7 +1971,7 @@ async def get_buvid() -> Tuple[str, str]:
     return (__buvid3, __buvid4)
 
 
-async def get_bili_ticket(credential: Optional[Credential] = None) -> str:
+async def get_bili_ticket(credential: Optional[Credential] = None) -> Tuple[str, str]:
     """
     获取 bili_ticket
 
@@ -1898,17 +1979,20 @@ async def get_bili_ticket(credential: Optional[Credential] = None) -> str:
         credential (Credential, optional): 凭据. Defaults to None.
 
     Returns:
-        str: bili_ticket
+        Tuple[str, str]: bili_ticket, bili_ticket_expires
     """
-    global __bili_ticket
+    global __bili_ticket, __bili_ticket_expires
+    if time.time() > int(__bili_ticket_expires):
+        refresh_bili_ticket()
     if __bili_ticket == "":
         __bili_ticket = await _get_bili_ticket(credential)
+        __bili_ticket_expires = str(int(time.time()) + 3 * 86400)
         request_log.dispatch(
             "ANTI_SPIDER",
             "反爬虫",
             {"msg": f"获取 bili_ticket 成功: [{__bili_ticket}]"},
         )
-    return __bili_ticket
+    return __bili_ticket, __bili_ticket_expires
 
 
 async def get_wbi_mixin_key(credential: Optional[Credential] = None) -> str:
@@ -1948,8 +2032,6 @@ class Api:
 
         wbi2 (bool, optional): 是否使用参数进一步的 wbi 鉴权. Defaults to False.
 
-        bili_ticket (bool, optional): 是否使用 bili_ticket. Defaults to False.
-
         verify (bool, optional): 是否验证凭据. Defaults to False.
 
         no_csrf (bool, optional): 是否不使用 csrf. Defaults to False.
@@ -1970,7 +2052,6 @@ class Api:
     comment: str = ""
     wbi: bool = False
     wbi2: bool = False
-    bili_ticket: bool = False
     verify: bool = False
     no_csrf: bool = False
     json_body: bool = False
@@ -2074,13 +2155,14 @@ class Api:
             self.data["csrf_token"] = self.credential.bili_jct
         # 处理 cookies
         cookies = self.credential.get_cookies()
-        if cookies["buvid3"] == "":
-            cookies["buvid3"] = (await get_buvid())[0]
+        if (cookies["buvid3"] == "" or cookies["buvid4"] == "") and request_settings.get_enable_auto_buvid():
+            buvids = await get_buvid()
+            cookies["buvid3"] = buvids[0]
+            cookies["buvid4"] = buvids[1]
         cookies["opus-goback"] = "1"
         # bili_ticket
-        if self.bili_ticket:
-            cookies["bili_ticket"] = await get_bili_ticket(self.credential)
-            cookies["bili_ticket_expires"] = str(int(time.time()) + 2 * 86400)
+        if request_settings.get_enable_bili_ticket():
+            cookies["bili_ticket"], cookies["bili_ticket_expires"] = await get_bili_ticket(self.credential)
         # APP 鉴权
         if self.sign:
             if self.method in ["POST", "DELETE", "PATCH"]:
@@ -2199,8 +2281,8 @@ class Api:
                 return await self._request(raw=raw, byte=byte)
             except ResponseCodeException as e:
                 # -403 时尝试重新获取 wbi_mixin_key 可能过期了
-                if e.code == -403:
-                    refresh_wbi_mixin_key()
+                if e.code == -403 and self.wbi:
+                    recalculate_wbi()
                     continue
                 # 不是 -403 错误直接报错
                 raise e
@@ -2216,4 +2298,29 @@ class Api:
         return await self.request()
 
 
+async def bili_simple_download(url: str, out: str, intro: str):
+    """
+    适用于下载 bilibili 链接的简易终端下载函数
+
+    默认会携带 HEADERS 访问链接，避免 403
+
+    用途举例：下载 video.get_download_url 返回结果中的链接
+
+    Args:
+        url   (str): 链接
+        out   (str): 输出地址
+        intro (str): 下载简述
+    """
+    dwn_id = await get_client().download_create(url, HEADERS)
+    bts = 0
+    tot = get_client().download_content_length(dwn_id)
+    with open(out, "wb") as file:
+        while True:
+            bts += file.write(await get_client().download_chunk(dwn_id))
+            print(f"{intro} - {out} [{bts} / {tot}]", end="\r")
+            if bts == tot:
+                break
+    print()
+
+
 ################################################## END Api ##################################################
